@@ -169,6 +169,7 @@ $string['cannotuseadmin'] = 'You need to be an admin user to use this page';
 $string['cannotuseadminadminorteacher'] = 'You need to be a teacher or admin user to use this page';
 $string['cannotusepage'] = 'Only teachers and administrators can use this page';
 $string['cannotusepage2'] = 'Sorry, you may not use this page';
+$string['cannotviewcategory'] = 'You don\'t have permission to view this list of courses.';
 $string['cannotviewprofile'] = 'You cannot view the profile of this user';
 $string['cannotviewreport'] = 'You cannot view this report';
 $string['cannotwritefile'] = 'Cannot write to file ({$a})';
@@ -185,18 +186,20 @@ $string['couldnotupdatenoexistinguser'] = 'Cannot update the user - user doesn\'
 $string['couldnotverifyagedigitalconsent'] = 'An error occurred while trying to verify the age of digital consent.<br />Please contact administrator.';
 $string['countriesphpempty'] = 'Error: The file countries.php in language pack {$a} is empty or missing.';
 $string['coursedoesnotbelongtocategory'] = 'The course doesn\'t belong to this category';
-$string['courseformatnotfound'] = 'The course format \'{$a}\' doesn\'t exist or is not recognized';
+$string['courseformatnotfound'] = 'The course format \'{$a}\' doesn\'t exist or is not recognised.';
 $string['coursegroupunknown'] = 'Course corresponding to group {$a} not specified';
-$string['courseidnotfound'] = 'Course id doesn\'t exist';
+$string['courseidnotfound'] = 'The course ID doesn\'t exist.';
 $string['courseidnumbertaken'] = 'ID number is already used for another course ({$a})';
 $string['coursemisconf'] = 'Course is misconfigured';
 $string['courserequestdisabled'] = 'Sorry, but course requests have been disabled by the administrator.';
 $string['csvcolumnduplicates'] = 'Duplicate columns detected';
 $string['csvemptyfile'] = 'The CSV file is empty';
+$string['csvfileerror'] = 'There is something wrong with the format of the CSV file. Please check the number of headings and columns match, and that the delimiter and file encoding are correct: {$a}';
 $string['csvfewcolumns'] = 'Not enough columns, please verify the delimiter setting';
 $string['csvinvalidcols'] = '<b>Invalid CSV file:</b> First line must include "Header Fields" and the file must be type of <br />"Expanded Fields/Comma Separated"<br />or<br /> "Expanded Fields with CAVV Result Code/Comma Separated"';
 $string['csvinvalidcolsnum'] = 'Invalid CSV file - each line must include 49 or 70 fields';
 $string['csvloaderror'] = 'An error occurred while loading the CSV file: {$a}';
+$string['csvnodata'] = 'Invalid CSV file - The CSV file has headers but does not contain any data.';
 $string['csvweirdcolumns'] = 'Invalid CSV file format - number of columns is not constant!';
 $string['dbconnectionfailed'] = '<p>Error: Database connection failed</p>
 <p>It is possible that the database is overloaded or otherwise not running properly.</p>
@@ -219,6 +222,7 @@ $string['ddlxmlfileerror'] = 'XML database file errors found';
 $string['destinationcmnotexit'] = 'The destination course module does not exist';
 $string['detectedbrokenplugin'] = 'Plugin "{$a}" is defective or outdated, can not continue, sorry.';
 $string['dmlexceptiononinstall'] = '<p>A database error has occurred [{$a->errorcode}].<br />{$a->debuginfo}</p>';
+$string['dmlparseexception'] = 'Error parsing SQL query';
 $string['dmlreadexception'] = 'Error reading from database';
 $string['dmltransactionexception'] = 'Database transaction error';
 $string['dmlwriteexception'] = 'Error writing to database';
@@ -262,6 +266,7 @@ $string['filternotactive'] = 'Filter {$a} is not currently active';
 $string['filternotenabled'] = 'Filter not enabled!';
 $string['filternotinstalled'] = 'Filter {$a} is not currently installed';
 $string['forumblockingtoomanyposts'] = 'You have exceeded the posting threshold set for this forum';
+$string['functionalityremoved'] = 'You are trying to access functionality that has been removed.';
 $string['generalexceptionmessage'] = 'Exception - {$a}';
 $string['gradepubdisable'] = 'Grade publishing disabled';
 $string['gradesneedregrading'] = 'The course grades need to be recalculated';
@@ -302,10 +307,10 @@ $string['invalidarguments'] = 'No valid arguments supplied';
 $string['invalidblockinstance'] = 'Invalid block instance for: {$a}';
 $string['invalidbulkenrolop'] = 'Invalid bulk enrolment operation requested.';
 $string['invalidcategory'] = 'Incorrect category!';
-$string['invalidcategoryid'] = 'Incorrect category id!';
+$string['invalidcategoryid'] = 'Incorrect category ID!';
 $string['invalidcomment'] = 'Comment is incorrect';
-$string['invalidcommentid'] = 'Invalid comment id';
-$string['invalidcommentitemid'] = 'Invalid comment itemid';
+$string['invalidcommentid'] = 'Invalid comment ID';
+$string['invalidcommentitemid'] = 'Invalid comment item ID';
 $string['invalidcommentarea'] = 'Invalid comment area';
 $string['invalidcommentparam'] = 'Invalid comment parameters';
 $string['invalidcomponent'] = 'Invalid component name';
@@ -320,7 +325,7 @@ $string['invalidcoursenameshort'] = 'Invalid short course name';
 $string['invaliddata'] = 'Data submitted is invalid';
 $string['invaliddatarootpermissions'] = 'Invalid permissions detected when trying to create a directory. Turn debugging on for further details.';
 $string['invaliddevicetype'] = 'Invalid device type';
-$string['invalidelementid'] = 'Incorrect element id!';
+$string['invalidelementid'] = 'Incorrect element ID!';
 $string['invalidentry'] = 'This is not valid entry!';
 $string['invalidevent'] = 'Invalid event';
 $string['invalidfieldname'] = '"{$a}" is not a valid field name';
@@ -328,10 +333,10 @@ $string['invalidfiletype'] = '"{$a}" is not a valid file type';
 $string['invalidformatpara'] = 'Incorrect format for choose parameter';
 $string['invalidformdata'] = 'Incorrect form data';
 $string['invalidfunction'] = 'Incorrect function';
-$string['invalidgradeitemid'] = 'Incorrect grade item id';
-$string['invalidgroupid'] = 'Incorrect group id specified';
+$string['invalidgradeitemid'] = 'Incorrect grade item ID';
+$string['invalidgroupid'] = 'Incorrect group ID specified';
 $string['invalidipformat'] = 'Invalid IP address format';
-$string['invaliditemid'] = 'Incorrect item id';
+$string['invaliditemid'] = 'Incorrect item ID';
 $string['invalidkey'] = 'Incorrect key';
 $string['invalidlegacy'] = 'Incorrect legacy role definition for type: {$a}';
 $string['invalidmd5'] = 'The check variable was wrong - try again';
@@ -341,7 +346,7 @@ $string['invalidmoduleid'] = 'Invalid module ID: {$a}';
 $string['invalidmodulename'] = 'Invalid module name: {$a}';
 $string['invalidnum'] = 'Invalid numeric value';
 $string['invalidnumkey'] = '$conditions array may not contain numeric keys, please fix the code!';
-$string['invalidoutcome'] = 'Incorrect outcome id';
+$string['invalidoutcome'] = 'Incorrect outcome ID';
 $string['invalidpagesize'] = 'Invalid page size';
 $string['invalidpasswordpolicy'] = 'Invalid password policy';
 $string['invalidpaymentmethod'] = 'Invalid payment method: {$a}';
@@ -352,15 +357,15 @@ $string['invalidrecordunknown'] = 'Can\'t find data record in database.';
 $string['invalidrequest'] = 'Invalid request';
 $string['invalidrole'] = 'Invalid role';
 $string['invalidroleid'] = 'Invalid role ID';
-$string['invalidscaleid'] = 'Incorrect scale id';
+$string['invalidscaleid'] = 'Incorrect scale ID';
 $string['invalidsection'] = 'Course module record contains invalid section';
 $string['invalidsesskey'] = 'Your session has most likely timed out. Please log in again.';
-$string['invalidshortname'] = 'That\'s an invalid short course name';
+$string['invalidshortname'] = 'Invalid short course name';
 $string['invalidstatedetected'] = 'Something has gone wrong: {$a}. This should never normally happen.';
 $string['invalidsourcefield'] = 'Draft file\'s source field is invalid';
 $string['invalidurl'] = 'Invalid URL';
 $string['invaliduser'] = 'Invalid user';
-$string['invaliduserid'] = 'Invalid user id';
+$string['invaliduserid'] = 'Invalid user ID';
 $string['invaliduserfield'] = 'Invalid user field: {$a}';
 $string['invaliduserdata'] = 'Invalid user data: {$a}';
 $string['invalidusername'] = 'The given username contains invalid characters';
@@ -432,7 +437,7 @@ $string['noparticipants'] = 'No participants found for this course';
 $string['noparticipatorycms'] = 'Sorry, but you have no participatory course modules to report on';
 $string['nopermissions'] = 'Sorry, but you do not currently have permissions to do that ({$a}).';
 $string['nopermissiontocomment'] = 'You can\'t add comments';
-$string['nopermissiontodelentry'] = 'You can\'t delete other people\'s entries!';
+$string['nopermissiontodelentry'] = 'You can\'t delete this comment!';
 $string['nopermissiontoeditcomment'] = 'You can\'t edit other people\'s comments!';
 $string['nopermissiontohide'] = 'No permission to hide!';
 $string['nopermissiontoimportact'] = 'You do not have the required permissions to import activities to this course';
@@ -495,6 +500,7 @@ $string['sendmessage'] = 'Send message';
 $string['serverconnection'] = 'Error connecting to the server';
 $string['servicedonotexist'] = 'The service does not exist';
 $string['sessionwaiterr'] = 'Timed out while waiting for session lock.<br />Wait for your current requests to finish and try again later.';
+$string['sessionexpired'] = 'Session expired';
 $string['sessioncookiesdisable'] = 'Incorrect use of require_key_login() - session cookies must be disabled!';
 $string['sessiondiskfull'] = 'The session partition is full. It is not possible to log in at this time. Please notify the server administrator.';
 $string['sessionhandlerproblem'] = 'Session handler is misconfigured';
