@@ -1,55 +1,58 @@
 <?php
 
 /**
-  V4.98 13 Feb 2008  (c) 2000-2008 John Lim (jlim#natsoft.com.my). All rights reserved.
-  Released under both BSD license and Lesser GPL library license. 
-  Whenever there is any discrepancy between the two licenses, 
+  @version   v5.20.14  06-Jan-2019
+  @copyright (c) 2000-2013 John Lim (jlim#natsoft.com). All rights reserved.
+  @copyright (c) 2014      Damien Regad, Mark Newnham and the ADOdb community
+  Released under both BSD license and Lesser GPL library license.
+  Whenever there is any discrepancy between the two licenses,
   the BSD license will take precedence.
-	
+
   Set tabs to 4 for best viewing.
- 
+
 */
 
 // security - hide paths
 if (!defined('ADODB_DIR')) die();
 
 class ADODB2_access extends ADODB_DataDict {
-	
+
 	var $databaseType = 'access';
 	var $seqField = false;
-	
- 	
+
+
  	function ActualType($meta)
 	{
 		switch($meta) {
 		case 'C': return 'TEXT';
 		case 'XL':
-		case 'X': return 'MEMO'; 
-		
+		case 'X': return 'MEMO';
+
 		case 'C2': return 'TEXT'; // up to 32K
 		case 'X2': return 'MEMO';
-		
+
 		case 'B': return 'BINARY';
-			
+
+		case 'TS':
 		case 'D': return 'DATETIME';
 		case 'T': return 'DATETIME';
-		
+
 		case 'L': return 'BYTE';
 		case 'I': return 'INTEGER';
 		case 'I1': return 'BYTE';
 		case 'I2': return 'SMALLINT';
 		case 'I4': return 'INTEGER';
 		case 'I8': return 'INTEGER';
-		
+
 		case 'F': return 'DOUBLE';
 		case 'N': return 'NUMERIC';
 		default:
 			return $meta;
 		}
 	}
-	
+
 	// return string must begin with space
-	function _CreateSuffix($fname, &$ftype, $fnotnull,$fdefault,$fautoinc,$fconstraint)
+	function _CreateSuffix($fname, &$ftype, $fnotnull,$fdefault,$fautoinc,$fconstraint,$funsigned)
 	{
 		if ($fautoinc) {
 			$ftype = 'COUNTER';
@@ -65,31 +68,28 @@ class ADODB2_access extends ADODB_DataDict {
 		if ($fconstraint) $suffix .= ' '.$fconstraint;
 		return $suffix;
 	}
-	
+
 	function CreateDatabase($dbname,$options=false)
 	{
 		return array();
 	}
-	
-	
+
+
 	function SetSchema($schema)
 	{
 	}
 
-	function AlterColumnSQL($tabname, $flds)
+	function AlterColumnSQL($tabname, $flds, $tableflds='',$tableoptions='')
 	{
 		if ($this->debug) ADOConnection::outp("AlterColumnSQL not supported");
 		return array();
 	}
-	
-	
-	function DropColumnSQL($tabname, $flds)
+
+
+	function DropColumnSQL($tabname, $flds, $tableflds='',$tableoptions='')
 	{
 		if ($this->debug) ADOConnection::outp("DropColumnSQL not supported");
 		return array();
 	}
-	
+
 }
-
-
-?>
\ No newline at end of file
