@@ -12,6 +12,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
+/* global YUI */
 
 /**
  * @module moodle-editor_atto-plugin
@@ -32,21 +33,22 @@ var MENUTEMPLATE = '' +
             'tabindex="-1" ' +
             'type="button" ' +
             'title="{{title}}">' +
-            '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" '+
-                'style="background-color:{{config.menuColor}};" src="{{config.iconurl}}" />' +
-            '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" src="{{image_url "t/expanded" "moodle"}}"/>' +
+            '<span class="editor_atto_menu_icon"></span>' +
+            '<span class="editor_atto_menu_expand"></span>' +
         '</button>';
 
 var DISABLED = 'disabled',
     HIGHLIGHT = 'highlight',
     LOGNAME = 'moodle-editor_atto-editor-plugin',
     CSS = {
-        EDITORWRAPPER: '.editor_atto_content'
+        EDITORWRAPPER: '.editor_atto_content',
+        MENUICON: '.editor_atto_menu_icon',
+        MENUEXPAND: '.editor_atto_menu_expand'
     };
 
 function EditorPluginButtons() {}
 
-EditorPluginButtons.ATTRS= {
+EditorPluginButtons.ATTRS = {
 };
 
 EditorPluginButtons.prototype = {
@@ -176,6 +178,7 @@ EditorPluginButtons.prototype = {
      * specified, in the class for the button.
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @return {Node} The Node representing the newly created button.
      */
     addButton: function(config) {
@@ -206,11 +209,14 @@ EditorPluginButtons.prototype = {
         var title = M.util.get_string(config.title, 'atto_' + pluginname);
 
         // Create the actual button.
-        button = Y.Node.create('<button class="' + buttonClass + '"' +
-                'tabindex="-1">' +
-                    '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" src="' + config.iconurl + '"/>' +
-                '</button>');
+        button = Y.Node.create('<button type="button" class="' + buttonClass + '"' +
+                'tabindex="-1"></button>');
         button.setAttribute('title', title);
+        window.require(['core/templates'], function(Templates) {
+            Templates.renderPix(config.icon, config.iconComponent, title).then(function(iconhtml) {
+                button.append(iconhtml);
+            });
+        });
 
         // Append it to the group.
         group.append(button);
@@ -281,7 +287,8 @@ EditorPluginButtons.prototype = {
     /**
      * Add a basic button which ties into the execCommand.
      *
-     * See {{#crossLink "M.editor_atto.EditorPluginButtons/addButton:method"}}addButton{{/crossLink}} for full details of the optional parameters.
+     * See {{#crossLink "M.editor_atto.EditorPluginButtons/addButton:method"}}addButton{{/crossLink}}
+     * for full details of the optional parameters.
      *
      * @method addBasicButton
      * @param {object} config The button configuration
@@ -337,6 +344,7 @@ EditorPluginButtons.prototype = {
      * specified, in the class for the button.
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @param {array} config.entries List of menu entries with the string (entry.text) and the handlers (entry.handler).
      * @param {number} [config.overlayWidth=14] The width of the menu. This will be suffixed with the 'em' unit.
      * @param {string} [config.menuColor] menu icon background color
@@ -377,6 +385,15 @@ EditorPluginButtons.prototype = {
             title: title
         }));
 
+        window.require(['core/templates'], function(Templates) {
+            Templates.renderPix(config.icon, config.iconComponent, title).then(function(iconhtml) {
+                button.one(CSS.MENUICON).append(iconhtml);
+            });
+            Templates.renderPix('t/expanded', 'core', '').then(function(iconhtml) {
+                button.one(CSS.MENUEXPAND).append(iconhtml);
+            });
+        });
+
         // Append it to the group.
         group.append(button);
 
@@ -510,7 +527,6 @@ EditorPluginButtons.prototype = {
         // Cancel the hide menu prevention.
         menuDialogue.set('preventHideMenu', false);
 
-        console.log('Menu item chosen');
         // Set the focus after hide so that focus is returned to the editor and changes are made correctly.
         menuDialogue.set('focusAfterHide', this.get('host').editor);
         menuDialogue.hide(e);
@@ -523,6 +539,7 @@ EditorPluginButtons.prototype = {
      * @param {object} config
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @param {object} [inheritFrom] A parent configuration that this configuration may inherit from.
      * @return {object} The normalized configuration
      * @private
@@ -538,9 +555,18 @@ EditorPluginButtons.prototype = {
             inheritFrom = {};
         }
 
+
+        // First we wrap the callback in function to handle formating of text inserted into collapsed selection.
+        config.inlineFormat = config.inlineFormat || inheritFrom.inlineFormat;
+        config._inlineCallback = config.callback || inheritFrom.callback;
+        config._callback = config.callback || inheritFrom.callback;
+        if (config.inlineFormat && typeof config._inlineCallback === 'function') {
+            config._callback = function(e, args) {
+                this.get('host').applyFormat(e, config._inlineCallback, this, args);
+            };
+        }
         // We wrap the callback in function to prevent the default action, check whether the editor is
         // active and focus it, and then mark the field as updated.
-        config._callback = config.callback || inheritFrom.callback;
         config.callback = Y.rbind(this._callbackWrapper, this, config._callback, config.callbackArgs);
 
         config._callbackNormalized = true;
@@ -562,7 +588,7 @@ EditorPluginButtons.prototype = {
     _normalizeIcon: function(config) {
         if (!config.iconurl) {
             // The default icon component.
-            if (!config.iconComponent) {
+            if (!config.iconComponent || config.iconComponent == 'moodle') {
                 config.iconComponent = 'core';
             }
             config.iconurl = M.util.image_url(config.icon, config.iconComponent);
@@ -602,7 +628,11 @@ EditorPluginButtons.prototype = {
             return;
         }
 
-        if (!this.get('host').isActive()) {
+        if (!(YUI.Env.UA.android || this.get('host').isActive())) {
+            // We must not focus for Android here, even if the editor is not active because the keyboard auto-completion
+            // changes the cursor position.
+            // If we save that change, then when we restore the change later we get put in the wrong place.
+            // Android is fine to save the selection without the editor being in focus.
             this.get('host').focus();
         }
 
@@ -630,7 +660,11 @@ EditorPluginButtons.prototype = {
      * The keyConfig will take either an array of keyConfigurations, in
      * which case _addKeyboardListener is called multiple times; an object
      * containing an optional eventtype, optional container, and a set of
-     * keyCodes, or just a string containing the keyCodes.
+     * keyCodes, or just a string containing the keyCodes. When keyConfig is
+     * not an object, it is wrapped around a function that ensures that
+     * only the expected key modifiers were used. For instance, it checks
+     * that space+ctrl is not triggered when the user presses ctrl+shift+space.
+     * When using an object, the developer should check that manually.
      *
      * @method _addKeyboardListener
      * @param {function} callback
@@ -644,7 +678,9 @@ EditorPluginButtons.prototype = {
     _addKeyboardListener: function(callback, keyConfig, buttonName) {
         var eventtype = 'key',
             container = CSS.EDITORWRAPPER,
-            keys;
+            keys,
+            handler,
+            modifier;
 
         if (Y.Lang.isArray(keyConfig)) {
             // If an Array was specified, call the add function for each element.
@@ -665,19 +701,27 @@ EditorPluginButtons.prototype = {
 
             // Must be specified.
             keys = keyConfig.keyCodes;
+            handler = callback;
 
         } else {
-            keys = this._getKeyEvent() + keyConfig + this._getDefaultMetaKey();
+            modifier = this._getDefaultMetaKey();
+            keys = this._getKeyEvent() + keyConfig + '+' + modifier;
             if (typeof this._primaryKeyboardShortcut[buttonName] === 'undefined') {
                 this._primaryKeyboardShortcut[buttonName] = this._getDefaultMetaKeyDescription(keyConfig);
             }
 
+            // Wrap the callback into a handler to check if it uses the specified modifiers, not more.
+            handler = Y.bind(function(modifiers, e) {
+                if (this._eventUsesExactKeyModifiers(modifiers, e)) {
+                    callback.apply(this, [e]);
+                }
+            }, this, [modifier]);
         }
 
         this._buttonHandlers.push(
             this.editor.delegate(
                 eventtype,
-                callback,
+                handler,
                 keys,
                 container,
                 this
@@ -688,6 +732,34 @@ EditorPluginButtons.prototype = {
                 'debug', LOGNAME);
     },
 
+    /**
+     * Checks if a key event was strictly defined for the modifiers passed.
+     *
+     * @method _eventUsesExactKeyModifiers
+     * @param  {Array} modifiers List of key modifiers to check for (alt, ctrl, meta or shift).
+     * @param  {EventFacade} e The event facade.
+     * @return {Boolean} True if the event was stricly using the modifiers specified.
+     */
+    _eventUsesExactKeyModifiers: function(modifiers, e) {
+        var exactMatch = true,
+            hasKey;
+
+        if (e.type !== 'key') {
+            return false;
+        }
+
+        hasKey = Y.Array.indexOf(modifiers, 'alt') > -1;
+        exactMatch = exactMatch && ((e.altKey && hasKey) || (!e.altKey && !hasKey));
+        hasKey = Y.Array.indexOf(modifiers, 'ctrl') > -1;
+        exactMatch = exactMatch && ((e.ctrlKey && hasKey) || (!e.ctrlKey && !hasKey));
+        hasKey = Y.Array.indexOf(modifiers, 'meta') > -1;
+        exactMatch = exactMatch && ((e.metaKey && hasKey) || (!e.metaKey && !hasKey));
+        hasKey = Y.Array.indexOf(modifiers, 'shift') > -1;
+        exactMatch = exactMatch && ((e.shiftKey && hasKey) || (!e.shiftKey && !hasKey));
+
+        return exactMatch;
+    },
+
     /**
      * Determine if this plugin is enabled, based upon the state of it's buttons.
      *
@@ -825,9 +897,9 @@ EditorPluginButtons.prototype = {
      */
     _getDefaultMetaKey: function() {
         if (Y.UA.os === 'macintosh') {
-            return '+meta';
+            return 'meta';
         } else {
-            return '+ctrl';
+            return 'ctrl';
         }
     },
 
