@@ -108,7 +108,11 @@ class oauth_helper {
             $this->access_token_secret = $args['access_token_secret'];
         }
         $this->http = new curl(array('debug'=>false));
-        $this->http_options = array();
+        if (!empty($args['http_options'])) {
+            $this->http_options = $args['http_options'];
+        } else {
+            $this->http_options = array();
+        }
     }
 
     /**
@@ -184,9 +188,6 @@ class oauth_helper {
         $oauth_params['oauth_nonce']	    = $this->get_nonce();
         $oauth_params['oauth_timestamp']    = $this->get_timestamp();
         $oauth_params['oauth_consumer_key'] = $this->consumer_key;
-        if (!empty($this->oauth_callback)) {
-            $oauth_params['oauth_callback'] = $this->oauth_callback->out(false);
-        }
         $oauth_params['oauth_signature_method']	= 'HMAC-SHA1';
         $oauth_params['oauth_signature']	= $this->sign($http_method, $url, $oauth_params, $this->sign_secret);
         return $oauth_params;
@@ -221,23 +222,25 @@ class oauth_helper {
      */
     public function request_token() {
         $this->sign_secret = $this->consumer_secret.'&';
-        $params = $this->prepare_oauth_parameters($this->request_token_api, array(), 'GET');
+
+        if (empty($this->oauth_callback)) {
+            $params = [];
+        } else {
+            $params = ['oauth_callback' => $this->oauth_callback->out(false)];
+        }
+
+        $params = $this->prepare_oauth_parameters($this->request_token_api, $params, 'GET');
         $content = $this->http->get($this->request_token_api, $params, $this->http_options);
         // Including:
         //     oauth_token
         //     oauth_token_secret
         $result = $this->parse_result($content);
         if (empty($result['oauth_token'])) {
-            throw new moodle_exception('Error while requesting an oauth token');
-        }
-        // build oauth authrize url
-        if (!empty($this->oauth_callback)) {
-            // url must be rawurlencode
-            $result['authorize_url'] = $this->authorize_url . '?oauth_token='.$result['oauth_token'].'&oauth_callback='.rawurlencode($this->oauth_callback->out(false));
-        } else {
-            // no callback
-            $result['authorize_url'] = $this->authorize_url . '?oauth_token='.$result['oauth_token'];
+            throw new moodle_exception('oauth1requesttoken', 'core_error', '', null, $content);
         }
+        // Build oauth authorize url.
+        $result['authorize_url'] = $this->authorize_url . '?oauth_token='.$result['oauth_token'];
+
         return $result;
     }
 
@@ -266,6 +269,11 @@ class oauth_helper {
         unset($params['oauth_callback']);
         $content = $this->http->post($this->access_token_api, $params, $this->http_options);
         $keys = $this->parse_result($content);
+
+        if (empty($keys['oauth_token']) || empty($keys['oauth_token_secret'])) {
+            throw new moodle_exception('oauth1accesstoken', 'core_error', '', null, $content);
+        }
+
         $this->set_access_token($keys['oauth_token'], $keys['oauth_token_secret']);
         return $keys;
     }
@@ -383,16 +391,24 @@ class oauth_helper {
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 abstract class oauth2_client extends curl {
-    /** var string client identifier issued to the client */
+    /** @var string $clientid client identifier issued to the client */
     private $clientid = '';
-    /** var string The client secret. */
+    /** @var string $clientsecret The client secret. */
     private $clientsecret = '';
-    /** var moodle_url URL to return to after authenticating */
+    /** @var moodle_url $returnurl URL to return to after authenticating */
     private $returnurl = null;
-    /** var string scope of the authentication request */
-    private $scope = '';
-    /** var stdClass access token object */
-    private $accesstoken = null;
+    /** @var string $scope of the authentication request */
+    protected $scope = '';
+    /** @var stdClass $accesstoken access token object */
+    protected $accesstoken = null;
+    /** @var string $refreshtoken refresh token string */
+    protected $refreshtoken = '';
+    /** @var string $mocknextresponse string */
+    private $mocknextresponse = '';
+    /** @var array $upgradedcodes list of upgraded codes in this request */
+    private static $upgradedcodes = [];
+    /** @var bool basicauth */
+    protected $basicauth = false;
 
     /**
      * Returns the auth url for OAuth 2.0 request
@@ -439,13 +455,28 @@ abstract class oauth2_client extends curl {
 
         // We have a token so we are logged in.
         if (isset($this->accesstoken->token)) {
-            return true;
+            // Check that the access token has all the requested scopes.
+            $scopemissing = false;
+            $scopecheck = ' ' . $this->accesstoken->scope . ' ';
+
+            $requiredscopes = explode(' ', $this->scope);
+            foreach ($requiredscopes as $requiredscope) {
+                if (strpos($scopecheck, ' ' . $requiredscope . ' ') === false) {
+                    $scopemissing = true;
+                    break;
+                }
+            }
+            if (!$scopemissing) {
+                return true;
+            }
         }
 
         // If we've been passed then authorization code generated by the
         // authorization server try and upgrade the token to an access token.
         $code = optional_param('oauth2code', null, PARAM_RAW);
-        if ($code && $this->upgrade_token($code)) {
+        // Note - sometimes we may call is_logged_in twice in the same request - we don't want to attempt
+        // to upgrade the same token twice.
+        if ($code && !in_array($code, self::$upgradedcodes) && $this->upgrade_token($code)) {
             return true;
         }
 
@@ -463,6 +494,15 @@ abstract class oauth2_client extends curl {
         return new moodle_url('/admin/oauth2callback.php');
     }
 
+    /**
+     * An additional array of url params to pass with a login request.
+     *
+     * @return array of name value pairs.
+     */
+    public function get_additional_login_parameters() {
+        return [];
+    }
+
     /**
      * Returns the login link for this oauth request
      *
@@ -471,15 +511,32 @@ abstract class oauth2_client extends curl {
     public function get_login_url() {
 
         $callbackurl = self::callback_url();
-        $url = new moodle_url($this->auth_url(),
-                        array('client_id' => $this->clientid,
-                              'response_type' => 'code',
-                              'redirect_uri' => $callbackurl->out(false),
-                              'state' => $this->returnurl->out_as_local_url(false),
-                              'scope' => $this->scope,
-                          ));
+        $params = array_merge(
+            [
+                'client_id' => $this->clientid,
+                'response_type' => 'code',
+                'redirect_uri' => $callbackurl->out(false),
+                'state' => $this->returnurl->out_as_local_url(false),
+                'scope' => $this->scope,
+            ],
+            $this->get_additional_login_parameters()
+        );
 
-        return $url;
+        return new moodle_url($this->auth_url(), $params);
+    }
+
+    /**
+     * Given an array of name value pairs - build a valid HTTP POST application/x-www-form-urlencoded string.
+     *
+     * @param array $params Name / value pairs.
+     * @return string POST data.
+     */
+    public function build_post_data($params) {
+        $result = [];
+        foreach ($params as $name => $value) {
+            $result[] = urlencode($name) . '=' . urlencode($value);
+        }
+        return implode('&', $result);
     }
 
     /**
@@ -490,18 +547,24 @@ abstract class oauth2_client extends curl {
      */
     public function upgrade_token($code) {
         $callbackurl = self::callback_url();
-        $params = array('client_id' => $this->clientid,
-            'client_secret' => $this->clientsecret,
+        $params = array('code' => $code,
             'grant_type' => 'authorization_code',
-            'code' => $code,
             'redirect_uri' => $callbackurl->out(false),
         );
 
+        if ($this->basicauth) {
+            $idsecret = urlencode($this->clientid) . ':' . urlencode($this->clientsecret);
+            $this->setHeader('Authorization: Basic ' . base64_encode($idsecret));
+        } else {
+            $params['client_id'] = $this->clientid;
+            $params['client_secret'] = $this->clientsecret;
+        }
+
         // Requests can either use http GET or POST.
         if ($this->use_http_get()) {
             $response = $this->get($this->token_url(), $params);
         } else {
-            $response = $this->post($this->token_url(), $params);
+            $response = $this->post($this->token_url(), $this->build_post_data($params));
         }
 
         if ($this->info['http_code'] !== 200) {
@@ -510,10 +573,22 @@ abstract class oauth2_client extends curl {
 
         $r = json_decode($response);
 
+        if (is_null($r)) {
+            throw new moodle_exception("Could not decode JSON token response");
+        }
+
+        if (!empty($r->error)) {
+            throw new moodle_exception($r->error . ' ' . $r->error_description);
+        }
+
         if (!isset($r->access_token)) {
             return false;
         }
 
+        if (isset($r->refresh_token)) {
+            $this->refreshtoken = $r->refresh_token;
+        }
+
         // Store the token an expiry time.
         $accesstoken = new stdClass;
         $accesstoken->token = $r->access_token;
@@ -521,6 +596,9 @@ abstract class oauth2_client extends curl {
             // Expires 10 seconds before actual expiry.
             $accesstoken->expires = (time() + ($r->expires_in - 10));
         }
+        $accesstoken->scope = $this->scope;
+        // Also add the scopes.
+        self::$upgradedcodes[] = $code;
         $this->store_token($accesstoken);
 
         return true;
@@ -538,9 +616,10 @@ abstract class oauth2_client extends curl {
      *
      * @param string $url The URL to request
      * @param array $options
+     * @param mixed $acceptheader mimetype (as string) or false to skip sending an accept header.
      * @return bool
      */
-    protected function request($url, $options = array()) {
+    protected function request($url, $options = array(), $acceptheader = 'application/json') {
         $murl = new moodle_url($url);
 
         if ($this->accesstoken) {
@@ -552,7 +631,15 @@ abstract class oauth2_client extends curl {
             }
         }
 
-        return parent::request($murl->out(false), $options);
+        if ($acceptheader) {
+            $this->setHeader('Accept: ' . $acceptheader);
+        }
+
+        $response = parent::request($murl->out(false), $options);
+
+        $this->resetHeader();
+
+        return $response;
     }
 
     /**
@@ -603,6 +690,15 @@ abstract class oauth2_client extends curl {
         }
     }
 
+    /**
+     * Get a refresh token!!!
+     *
+     * @return string
+     */
+    public function get_refresh_token() {
+        return $this->refreshtoken;
+    }
+
     /**
      * Retrieve a token stored.
      *
